#!/usr/bin/env python
# vim: set ts=4 sts=4 sw=4 textwidth=92:

import rsutil.common

import re
import os
import logging

# This is a fairly generic class which does all of the major things that a PAR2
# set will need to have done to be verified and extracted. For most "normal" types
# you won't need to override hardly anything.
#
# It is ok to override other functions if the need arises, just make sure that you
# understand why things are done the way that they are in the original versions.
#
# Assumptions made about each of the run*() functions:
# ==============================================================================
# The state of self.name_matched_files and self.prot_matched_files will be consistent
# with the real, in-filesystem state at the time that they are called.
# (This is why runAll() calls update_matches() all the time.)
#
# Required overrides:
# ==============================================================================
# find_extraction_heads ()
# extraction_function ()
#

class Base (object):

	# Instance Variables
	# ==========================================================================
	# dir					-- The directory this set lives in
	# p2file				-- The starting PAR2 file
	# basename				-- The basename of the set, guessed from the PAR2 file
	# all_p2files			-- All PAR2 files of the set, guessed from the PAR2 file name only
	# name_matched_files	-- Files in this set, guessed by name only
	# prot_matched_files	-- Files in this set, guessed by parsing the PAR2 only

	def __init__ (self, dir, p2file):
		assert os.path.isdir (dir)
		assert os.path.isfile (os.path.join (dir, p2file))

		# The real "meat" of the class
		self.dir = dir
		self.p2file = p2file
		self.basename = rsutil.common.get_basename (p2file)

		# Find files that match by name only
		self.name_matched_files = rsutil.common.find_name_matches (self.dir, self.basename)

		# Find all par2 files for this set using name matches
		self.all_p2files = rsutil.common.find_par2_files (self.name_matched_files)

		# Try to get the protected files for this set
		self.prot_matched_files = rsutil.common.parse_all_par2 (self.dir, self.p2file, self.all_p2files)

		# Setup the all_files combined set (for convenience only)
		self.all_files = rsutil.common.no_duplicates (self.name_matched_files + self.prot_matched_files)

	def __eq__ (self, rhs):
		return (self.dir == rhs.dir) and (self.basename == rhs.basename) and \
				rsutil.common.list_eq (self.name_matched_files, rhs.name_matched_files) and \
				rsutil.common.list_eq (self.prot_matched_files, rhs.prot_matched_files)

	def update_matches (self):
		"""Updates the contents of instance variables which are likely to change after
		running an operation, usually one which will create new files."""

		self.name_matched_files = rsutil.common.find_name_matches (self.dir, self.basename)
		self.all_files = rsutil.common.no_duplicates (self.name_matched_files + self.prot_matched_files)

	def runVerifyAndRepair (self):
		PAR2_CMD = rsutil.common.config_get_value ('commands', 'par2repair')

		# assemble the command
		# par2repair -- PAR2 PAR2_EXTRA [JOIN_FILES]
		command = "%s \"%s\" " % (PAR2_CMD, self.p2file)

		for f in self.all_p2files:
			if f != self.p2file:
				command += "\"%s\" " % os.path.split (f)[1]

		# run the command
		ret = rsutil.common.run_command (command, self.dir)

		# check the result
		if ret != 0:
			logging.critical ('PAR2 Check / Repair failed: %s' % self.p2file)
			return -rsutil.common.ECHECK

		return rsutil.common.SUCCESS

	def find_deleteable_files (self):
		DELETE_REGEX = rsutil.common.config_get_value ('regular expressions', 'delete_regex')
		dregex = re.compile (DELETE_REGEX, re.IGNORECASE)

		return [f for f in self.all_files if dregex.match (f)]

	def delete_list_of_files (self, dir, files, interactive=False):
		# Delete a list of files

		assert os.path.isdir (dir)

		done = False
		valid_y = ['Y', 'YES']
		valid_n = ['N', 'NO', '']

		if interactive:
			while not done:
				print 'Do you want to delete the following?:'
				for f in files:
					print f
				s = raw_input ('Delete [y/N]: ').upper()

				if s in valid_y + valid_n:
					done = True

			if s in valid_n:
				return rsutil.common.SUCCESS

		for f in files:
			try:
				os.remove (os.path.join (dir, f))
				logging.debug ('Deleteing: %s' % os.path.join (dir, f))
			except:
				logging.error ('Failed to delete: %s' % os.path.join (dir, f))
				return -rsutil.common.EDELETE

		return rsutil.common.SUCCESS

	def runDelete (self):
		deleteable_files = self.find_deleteable_files ()
		ret = self.delete_list_of_files (self.dir, deleteable_files, \
				rsutil.common.options_get_value ('interactive'))

		return ret

	def runAll (self):

		# Repair Stage
		ret = self.runVerifyAndRepair ()

		if ret != rsutil.common.SUCCESS:
			logging.critical ('Repair stage failed for: %s' % self.p2file)
			return -rsutil.common.ECHECK

		self.update_matches ()

		# Extraction Stage
		ret = self.runExtract ()

		if ret != rsutil.common.SUCCESS:
			logging.critical ('Extraction stage failed for: %s' % self.p2file)
			return -rsutil.common.EEXTRACT

		self.update_matches ()

		# Deletion Stage
		ret = self.runDelete ()

		if ret != rsutil.common.SUCCESS:
			logging.critical ('Deletion stage failed for: %s' % self.p2file)
			return -rsutil.common.EDELETE

		logging.info ('Successfully completed: %s' % self.p2file)
		return rsutil.common.SUCCESS

	def safe_create_directory (self, dir):
		if dir == None:
			return rsutil.common.SUCCESS

		if os.path.isdir (dir):
			return rsutil.common.SUCCESS

		try:
			os.makedirs (dir)
			logging.info ('Created directory: %s' % dir)
		except OSError:
			logging.critical ('FAILED to create directory: %s' % dir)
			return -rsutil.common.ECREATE

		return rsutil.common.SUCCESS

	def runExtract (self, todir=None):
		"""Extract all heads of this set"""

		# Extract to the head's dir if we don't care where to extract
		if todir == None:
			todir = self.dir

		# Create the directory $todir if it doesn't exist
		ret = self.safe_create_directory (todir)

		if ret != rsutil.common.SUCCESS:
			return -rsutil.common.EEXTRACT

		# Call the extraction function on each head
		for h in self.find_extraction_heads ():
			full_head = rsutil.common.full_abspath (os.path.join (self.dir, h))
			ret = self.extraction_function (full_head, todir)
			logging.debug ('Extraction Function returned: %d' % ret)

			# Check error code
			if ret != rsutil.common.SUCCESS:
				logging.critical ('Failed extracting: %s' % h)
				return -rsutil.common.EEXTRACT

		return rsutil.common.SUCCESS

	def find_extraction_heads (self):
		assert False # You MUST override this on a per-type basis

	def extraction_function (self, file, todir):
		# NOTE: Please keep the prototype the same for all overridden functions.
		# Doing so will guarantee that your life is made much easier.
		#
		# Also note that the todir given will always be valid for the current directory
		# when the function is called.

		assert False # You MUST override this on a per-type basis

