#!/usr/bin/env python
# vim: set ts=4 sts=4 sw=4 textwidth=92:

import rsutil.common

# PAR2Set-derived types
import PAR2Set.Join
import PAR2Set.ZIP
import PAR2Set.OldRAR
import PAR2Set.NewRAR
import PAR2Set.ExtractFirstOldRAR
import PAR2Set.ExtractFirstNewRAR

import logging

class RarslaveDetector (object):

	# A tuple of tuples with the following definition:
	# (TYPE_NAME, DETECTION_FUNCTION, PAR2Set-derived class)

	TYPES = (	(PAR2Set.Join.detector,   PAR2Set.Join.Join),
				(PAR2Set.ZIP.detector,    PAR2Set.ZIP.ZIP),
				(PAR2Set.OldRAR.detector, PAR2Set.OldRAR.OldRAR),
				(PAR2Set.NewRAR.detector, PAR2Set.NewRAR.NewRAR),
				(PAR2Set.ExtractFirstOldRAR.detector, PAR2Set.ExtractFirstOldRAR.ExtractFirstOldRAR),
				(PAR2Set.ExtractFirstNewRAR.detector, PAR2Set.ExtractFirstNewRAR.ExtractFirstNewRAR),
			)

	def __init__ (self, dir, p2file):

		# The real "meat" of the class
		self.dir = dir
		self.p2file = p2file
		self.basename = rsutil.common.get_basename (p2file)

		# Find files that match by name only
		self.name_matched_files = rsutil.common.find_name_matches (self.dir, self.basename)

		# Find all par2 files for this set using name matches
		self.all_p2files = rsutil.common.find_par2_files (self.name_matched_files)

		# Try to get the protected files for this set
		self.prot_matched_files = rsutil.common.parse_all_par2 (self.dir, self.p2file, self.all_p2files)

	def runMatchingTypes (self):
		# Now tries to run every type of PAR2Set-derived class for which the detector
		# detects that the class is valid.

		detected = False

		for (detector, classname) in self.TYPES:
			if detector (self.name_matched_files, self.prot_matched_files):
				# The detector matched, so we're up and running!

				p2set = classname (self.dir, self.p2file)

				detected = True
				logging.debug ('Detected type: %s' % p2set)

				# Try to have rarslave do it's thing
				ret = p2set.runAll ()

				# If something already worked, there is no need to continue,
				# since we've already finished!
				if ret == rsutil.common.SUCCESS:
					break
				else:
					logging.error ('Detected type failed for: %s' % self.p2file)

		# Make sure we detected at least one valid type
		if not detected:
			logging.warning ('Unable to determine type: %s' % self.p2file)
			logging.debug ('The following information will help in writing a detector:')
			logging.debug ('name_matches: %s' % self.name_matched_files)
			logging.debug ('prot_matches: %s' % self.prot_matched_files)
			return -rsutil.common.EDETECT

		# Make sure that something worked
		if ret != rsutil.common.SUCCESS:
			logging.critical ('All types failed for: %s' % self.p2file)

		return ret

def main ():
	pass

if __name__ == '__main__':
	main ()

